;       9852DDSX.ASM     G4JNT  October 2004
;Version of 9852DDS for 16F626 processor

;Controls AD9852 DDS with ASCII Hex commands via 19200 Baud serial interface

;########################################################
;#  ....Modified code to use 16F628 from 16F84......    #
;#     Set LV programming off to enable B4              #
;#     Set CMCOM bits to disable comparator on PORTA    #
;#     Change GetEE  as EE registers now all in bank 1  #
;#     Ext. EC Osc mode in config bits allows PORTA,6   #
;########################################################


;               Command Protocol
;           EEPROM Contents loaded at start up

;           Quick updates of Frequency and phase for datamodes
;               Qxxxxxxxxxxxx   [cr]    48 bit frequency word
;               Pxxxx           [cr]    16 bit phase word
;               Mzxxx           [cr]    12 bit Amplitude word (z = NU)
;               U                       Update DDS with Q, P or A data
;               T                       Update on next timer pulse
;               W                       Save Current frequency & amplitude to
;                                         EEPROM,  also updates DDS chip

;           General Chip Programming
;               G                   Update individual registers (via prompt)
;                                        send to 9852 and save to EEPROM
;               V                   Readback all EEProm contents

;           User data for readback (eg clock frequency)
;               K                   (via prompt) enter 15 ASCII chars of
;                                       user data, store straight to EEPROM
;               R                   Read User Data

        LIST P=16F628
        INCLUDE "p16F628.inc"
        errorlevel  1

        __config    0x3F61      ;LV Programming off , OSC XT Mode


;   EEPROM Contents, use chip defaults except freq 1 = 0.25Fc, Ampl = max
;                  and control register set for CW output, no PLL, ampl ON
    org 0x2100
        de  0,0            ;Addr 0  Phase offset Reg 1     2    0x00 - 0x01
        de  0,0                 ;1  Phase offset Reg 2     2    0x02 - 0x03
        de  0x40,0,0,0,0,0      ;2  Freq Tuning Word 1     6    0x04 - 0x09
        de  0,0,0,0,0,0         ;3  Freq Tuning Word 2     6    0x0A - 0x0F
        de  0,0,0,0,0,0         ;4  Delta Frequency        6    0x10 - 0x15
        de  0,0,0,0x40          ;5  Update Clock           4    0x16 - 0x19
        de  0,0,0               ;6  Ramp rate Clock        3    0x1A - 0x1C
        de  0x10,0x64,0x00,0x60 ;7  Control Register       4    0x1D - 0x20
        de  0x0F,0xFF           ;8  Digital Multiplier     2    0x21 - 0x22
        de  0,0                 ;9    Spare                2
        de  0                   ;A  Shaped On/Off keying   1    0x25
        de  0,0                 ;B  Control DAC register   2    0x26 - 0x27


    org 0x2130                  ; User data                     0x30 - 0x3F
        de  "240000000",0
                                
    cblock 0x20
        BitCount
        DelCount  
        DelCount1
        DelCount2   
        Temp
        RxChar
        SD        
        D0                      ;Working registers for storing/reading data
        D1    
        D2    
        D3     
        D4
        D5
        F0                      ;Independent frequency storage
        F1
        F2
        F3
        F4
        F5
        A0                      ;Indepentant amplitude storage
        A1
        Counter
        Counter2
        Flags
        AddrHold
        MsgCount
        DDSAddr
        AutoEEADR               ;To save flipping the page bit when using EE
                                 addressing
        Stack1
        Stack2
        Stack3
        Stack4
        Stack5
        Stack6
        Stack7
        Stack8
        Stack9
        StackA
        StackB
        StackC
        StackD
        StackE
        StackF
    endc

    #define     RXD         PORTB, 0    ;Data IN from PC
    #define     TXD         PORTB, 3    ;Data OUT to PC
    #define     LED         PORTB, 4
    #define     ExtTim      PORTB, 1    ;External timing pulse (eg. GPS)

    #define     MR          PORTA, 0
    #define     SCLK        PORTA, 1
    #define     IOUD        PORTA, 2
    #define     SDIO        PORTA, 3
    #define     IORST       PORTA, 4

    #define     FreqSet     Flags, 2    ;Used when saving freq to EE after Q
    #define     AmpSet      Flags, 3

;code starts here

    org     0
boot    nop
    clrw
    movwf   INTCON      ;disable interrupts
    goto    startup     ;jump to main code
;.....................................................
ints
    retfie           
;.....................................................
startup

    clrf    PORTA           ;Special setup to disable comparator on 'F628
    movlw   7
    movwf   CMCON


    bsf     STATUS,RP0  ;ram page 1
    movlw   b'11100000' ;
    movwf   PORTA       ;set all port A0 as outputs
    movlw   b'00000011' ;B3 RS232 out, B0 RS232 in, B1 External Timing
    movwf   PORTB       ;
    bcf     OPTION_REG , NOT_RBPU   ;enable pull ups
    bcf     STATUS,RP0  ;ram page 0

    movlw   b'00001000' ;PortB Change, but don't enable interrupts
    movwf   INTCON      ;RBIE only

    bsf     LED
    movlw   d'10'
    movwf   DelCount2

    clrf    PORTA
    bsf     MR  
StartDel1               ;270ms delay to let things stabilise with MR active
    movlw   0
    movwf   DelCount1
StartDel2
    call    Delay
    decfsz  DelCount1   ;256 calls to delay ...
    goto    StartDel2
    decfsz  DelCount2   ;delcount2 times
    goto    StartDel1

    bcf     MR
    clrf    DelCount1   ;Wait another 27ms after dropping MR
StartDel3               ;    to setup the DDS
    call    Delay
    decfsz  DelCount1  
    goto    StartDel3
    call    SetUpDDS
    bcf     LED

    movlw   LOW(IntroMsg)
    call    DispMsg
    call    CrLf
    movlw   LOW(CmdLst)
    call    DispMsg
    call    ReadBackEE
    call    CrLf
    call    ReadBackK
    call    CrLf
;---------------------------------------------------------------
MainLoop
    call    PushStack
    call    WaitData    ;Wait for RS232 character then push onto stack
    movwf   Stack1
                        ;Check for single letter commands
    movf    Stack1, W
    sublw   "U"           ;Update 9852 with Q or P data
    btfsc   STATUS, Z
    call    Write9852

    movf    Stack1, W
    sublw   "T"           ;Update 9852 with Q or P data on timer pulse
    btfsc   STATUS, Z
    call    WriteTim

    movf    Stack1, W
    sublw   "G"           ;Update individual registers
    btfsc   STATUS, Z
    goto    UpDateReg

    movf    Stack1, W
    sublw   "W"           ;Save current frequency to EEProm
    btfsc   STATUS, Z
    call    WriteEE

    movf    Stack1, W     ;Set user data in EEProm
    sublw   "K"
    btfsc   STATUS, Z
    goto    Kdata

    movf    Stack1, W     ;Read user data from EEProm
    sublw   "R"
    btfsc   STATUS, Z
    call    ReadBackK

    movf    Stack1, W     ;Read user data from EEProm
    sublw   "V"
    btfsc   STATUS, Z
    call    ReadBackEE

    movf    Stack1, W     ;[cr] terminates data entry so extract the data
    sublw   0x0D          ; eg.  K and Q commands, from stack
    btfsc   STATUS, Z
    goto    ReadStack

    goto    MainLoop      ;for P and Q commands
 ;...........
ReadStack    ;Here when [cr] appears     ED CB A9 87 65 43 2  1
                                        ; 5  4  3  2  1  0   [cr]

    movf    StackE, W               ;Frequency
    sublw   "Q"
    btfsc   STATUS, Z
    goto    Qdata

    movf    Stack6, W               ;Phase
    sublw   "P"
    btfsc   STATUS, Z
    goto    Pdata

    movf    Stack6, W               ;Amplitude
    sublw   "M"
    btfsc   STATUS, Z
    goto    Mdata

    goto    MainLoop
;........
Qdata           ;6 bytes of frequency register data
    call    StackToD
    call    DToF        ;Store frequency separately
    movlw   2
    movwf   DDSAddr

    call    DataTo9852
    call    DispDData
    call    ClrStack
    movlw   "Q"
    call    SendAck
    bsf     FreqSet
    goto    MainLoop
;........
Pdata           ;2 bytes of phase data
    call    StackToD        ;Transfer all stack contents to D, even rubbish
    movlw   0
    movwf   DDSAddr

    call    DataTo9852
    call    DispDData
    call    ClrStack
    movlw   "P"
    call    SendAck
    goto    MainLoop
;........
Mdata           ;4 bytes of amplitude data (MSN redundant)
    call    StackToD
    call    DToM
    movlw   8
    movwf   DDSAddr

    call    DataTo9852
    call    DispDData
    call    ClrStack
    movlw   "M"
    call    SendAck
    bsf     AmpSet
    goto    MainLoop
;........
Kdata           ;Up to 16 bytes of user data stored directly in EEPRom
    movlw   LOW(KPrompt)
    call    DispMsg

    movlw   0x30
    movwf   AutoEEADR

KDataLoop
    call    WaitData
    movlw   0x0D
    subwf   RxChar, W
    btfsc   STATUS, Z
    goto    OutKData

    movf    RxChar, W
    call    StoreEEW
    incf    AutoEEADR
    movlw   0x3F
    subwf   AutoEEADR, W        ;W = AutoEEADR - 0x3F, +ve/0 (C=1) when full
    btfss   STATUS, C
    goto    KDataLoop

    movlw   LOW(EEFull)
    call    DispMsg

OutKData
    movlw   0        ;Null terminated in last character
    call    StoreEEW
    call    ReadBackK

    goto    MainLoop
;.....................
UpDateReg
    movlw   LOW(RegPr1)
    call    DispMsg                 ;Prompt for register address
    call    WaitData
    call    DecodeHex
    movwf   DDSAddr
    movf    DDSAddr, W
    call    HexToAscii
    call    Send232         ;Echo back corrected HEX version to double check
    call    CrLf

    call    GetRegLen       ;Get number of bytes to be received, from DDSAddr
    movwf   Counter2
    addlw   0x30
    call    Send232
    movlw   LOW(BytesMsg)
    call    DispMsg

    bcf     STATUS, C
    rlf     Counter2        ;Multiply by 2
RegLoop
    call    PushStack
    call    WaitData
    movwf   Stack1
    decfsz  Counter2
    goto    RegLoop

    call    PushStack       ;To position correctly (Stack1 is not used)

    call    StackToD        ;Extract all stack contents, even if trash
    call    DataTo9852      ;Send appropriate number of bytes to AD9852
    call    DataToEE
    call    DispDData       ;This assumes DataTo9852 doesn't destroy D0-5
    call    CrLf

DoneRegData
    call    ClrStack
    goto    MainLoop
;=====================  SUBROUTINES BELOW HERE  ========================
StackToD                    ;Extract hex data from stack, decode and
    movf    StackD, W       ;  transfer binary data to D5/4/3/2/1/0
    call    DecodeHex
    movwf   D5              ;Stack contents  EDCBA98765432  1
    swapf   D5              ;  D registers   ~554433221100[cr]
    movf    StackC, W
    call    DecodeHex
    addwf   D5

    movf    StackB, W
    call    DecodeHex
    movwf   D4
    swapf   D4
    movf    StackA, W
    call    DecodeHex
    addwf   D4

    movf    Stack9, W
    call    DecodeHex
    movwf   D3
    swapf   D3
    movf    Stack8, W
    call    DecodeHex
    addwf   D3

    movf    Stack7, W
    call    DecodeHex
    movwf   D2
    swapf   D2
    movf    Stack6, W
    call    DecodeHex
    addwf   D2

    movf    Stack5, W
    call    DecodeHex
    movwf   D1
    swapf   D1
    movf    Stack4, W
    call    DecodeHex
    addwf   D1

    movf    Stack3, W
    call    DecodeHex
    movwf   D0
    swapf   D0
    movf    Stack2, W
    call    DecodeHex
    addwf   D0

    return
;--------------------------------
DToF                            ;Transfer D0-5 to F0-5 to preserve current
    movf    D0, W               ;  frequency for subsequent W command
    movwf   F0
    movf    D1, W
    movwf   F1
    movf    D2, W
    movwf   F2
    movf    D3, W
    movwf   F3
    movf    D4, W
    movwf   F4
    movf    D5, W
    movwf   F5
    return
;--------------------------------
DToM
    movf    D0, W               ;Preserve Amplitude for subsequent W command
    movwf   A0
    movf    D1, W
    movwf   A1
    return
;--------------------------------
ReadBackK
    movlw   0x30
    movwf   AutoEEADR
ReadKLoop
    call    AutoEE      ;version of GetEE for when AutoEEADR is set externally
    addlw   0
    btfsc   STATUS, Z
    goto    OutReadK

    call    Send232
    incf    AutoEEADR
    movlw   0x40
    subwf   AutoEEADR, W        ;W = AutoEEADR - 0x40, +ve/0 (C=1) when full
    btfss   STATUS, C
    goto    ReadKLoop

OutReadK
    call    CrLf
    return
;--------------------------------
ReadBackEE      ;Dump contents of EE registers in easily read format.
                ;Also Sends contents to AD9852
    clrf    DDSAddr         ;Count of Address registers
ReadEELoop
    call    GetEEAddr       ;EE Start address for each value of DDSAddr
    movwf   AutoEEADR
    
    movf    DDSAddr, W      ;Display each address
    call    Hex232
    movlw   " "
    call    Send232
    movlw   "="
    call    Send232
    movlw   " "
    call    Send232
    
    movf    DDSAddr, W
    call    Send9852Byte    ;Send address to AD9852
    call    GetRegLen       ;Get number of bytes for this address
    movwf   Counter2
RdEEDatLoop
    call    AutoEE          ;EE Data is stored MSB first, same as sent to
    call    Hex232          ; display and to AD9852 chip.  Views Ok in 
    movlw   " "             ; memory dump
    call    Send232

    call    AutoEE          ;Recover EE data to W
    call    Send9852Byte    ;Send to AD9852 MSB first

    incf    AutoEEADR
    decfsz  Counter2
    goto    RdEEDatLoop

    call    CrLf
    incf    DDSAddr
    movlw   0x0C
    subwf   DDSAddr, W
    btfss   STATUS, Z
    goto    ReadEELoop

    call    Write9852       ;Immediate update of chip with EEProm contents
    return
;--------------------------------
WaitData        ;Get character from RS232 input and convert to upper case
    btfss   RXD         ;look for start transition, low/high = 1/0
    goto    WaitData    ;
    movlw   d'7'        ; ( may need to adjust this)
    movwf   DelCount     ;
    nop
LoopStartBit            ;this loop 3.N - 1 long = 17
    decfsz  DelCount
    goto    LoopStartBit

    btfss   RXD         ;make sure it's still high, now centre of bit
    goto    WaitData      ;
    movlw   8           ;
    movwf   Counter
    clrf    RxChar      ;26us to here from transition = half bit
ByteLoop                
    call    BitDelay
    nop
    bcf     STATUS , C     
    btfss   RXD         
    bsf     STATUS , C     
    rrf     RxChar
    decfsz  Counter
    goto    ByteLoop      ;this loop BitDelay + 8

    movlw   "a"             ;Convert to Upper case
    subwf   RxChar, W       ;W = Char - "a"
    btfss   STATUS, C       ;if-ve (C=0) char < "a" so ignore
    goto    UpCaseDone

    movf    RxChar, W
    sublw   "z"             ;W = "z" - Char
    btfss   STATUS, C       ;if -ve (C=0)  char > "z" so ignore
    goto    UpCaseDone

    bcf     RxChar, 5       ;Reset bit 5 to force upper case
UpCaseDone
    movf    RxChar , W    
    return                  ;received byte in RxChar and W
;-------------------------
Hex232      ;Send a byte as hex data. Uses   RxChar [Temp, Counter, DelCount]
    movwf   RxChar          ;Use RxChar as temporary storage only
    swapf   RxChar, W       ;Send MS Nibble
    andlw   0x0F
    call    HexToAscii      
    call    Send232
    movf    RxChar, W       ;Send LS Nibble
    andlw   0x0F
    call    HexToAscii
    call    Send232
    return
;-------------------------
Send232                         ;Enter with data in W, send it to PC
    movwf   Temp            ;Uses Temp, Counter, [DelCount]
    bsf     TXD         ;start bit
    call    BitDelay    ;44us, need 52 total
    movlw   8
    movwf   Counter
    nop
    nop
    nop
SendDatLoop
    rrf     Temp        ;bit into C
    btfsc   STATUS , C
    bcf     TXD
    btfss   STATUS , C
    bsf     TXD     ;could have 2us jitter depending on 1/0
    call    BitDelay
    decfsz  Counter
    goto    SendDatLoop     ;loop 8 + BitDelay long
    bcf     TXD     ;stop bit
    call    BitDelay
    nop
    nop
    nop
    nop
    nop
    nop
    return
;--------------------------------------
BitDelay        ;Total incl call and return, (3N+5) = 44us   Uses Delcount
    movlw   d'13'         ;19200 = 52us, -8us used in Tx/Rx routines
    movwf   DelCount      ;baud period delay
LoopBit
    decfsz  DelCount        
    goto    LoopBit
    return

;-------------------------
DecodeHex           ;Single ASCII character to binary 
    movwf   Temp            ;Uses Temp
    movlw   0x30
    subwf   Temp            ;subtract $30 and leave in Temp
    btfss   STATUS , C      ;if < $30, result -ve so C=0, error, set=0
    clrf    Temp

    movf    Temp , W        
    sublw   d'9'            ;W = 9 - W
    btfsc   STATUS , C      ;if -ve, C=0, char > 9, so subtract 7
    goto    ConvDone
    movlw   d'7'
    subwf   Temp            ;
ConvDone
    movf    Temp , W        ;
    andlw   0x0F            ;mask out any rubbish left in
    return              
;-------------------------
HexToAscii          ;Uses Temp
    addlw   0x30
    movwf   Temp
    sublw   0x39
    btfsc   STATUS , C
    goto    AsciiDone
    movlw   7
    addwf   Temp
AsciiDone
    movf    Temp , W
    return
;-------------------------
WriteEE         ;Update and Write current frequency / amplitude to EE
    btfss   FreqSet
    goto    TestForAmp

    movlw   4          
    movwf   AutoEEADR
    movf    F5, W
    call    StoreEEW
    movlw   5
    movwf   AutoEEADR
    movf    F4, W
    call    StoreEEW
    movlw   6
    movwf   AutoEEADR
    movf    F3, W
    call    StoreEEW
    movlw   7
    movwf   AutoEEADR
    movf    F2, W
    call    StoreEEW
    movlw   8
    movwf   AutoEEADR
    movf    F1, W
    call    StoreEEW
    movlw   9
    movwf   AutoEEADR
    movf    F0, W
    call    StoreEEW
;..........
TestForAmp
    btfss   AmpSet
    goto    WDone

    movlw   0x21
    movwf   AutoEEADR
    movf    A1, W
    call    StoreEEW
    movlw   0x22
    movwf   AutoEEADR
    movf    A0, W
    call    StoreEEW

WDone
    call    Write9852
    movlw   "W"
    call    SendAck
    bcf     FreqSet
    bcf     AmpSet
    return    
;-------------------------
StoreEEW          ;Enter with W = EE data,  AutoEEADR = EE address
    bsf     STATUS, RP0
    movwf   EEDATA
    bcf     STATUS, RP0
    movf    AutoEEADR, W
    bsf     STATUS, RP0
    movwf   EEADR    

    bsf     STATUS , RP0    ;All EE registers in Bank 1

    bsf     EECON1 , WREN   ;Enable EEprom writing
    movlw   0x55
    movwf   EECON2
    movlw   0xAA
    movwf   EECON2
    bsf     EECON1 , WR

    bcf     STATUS, RP0     ;PIR1 register is in bank 0
EEwaitW
   
    btfss   PIR1 , EEIF
    goto    EEwaitW
    bcf     PIR1 , EEIF       ;has to be cleared manually

    bsf     STATUS, RP0
    bcf     EECON1 , WR
    bcf     STATUS , RP0

    return
;-------------------------
AutoEE                       
    movf    AutoEEADR, W
GetEE         ;different from ;F84   EE registers all in bank 1 now 
    bsf     STATUS,RP0      ;ram page 1
    movwf   EEADR
    bsf     EECON1 , RD
    movf    EEDATA , W
    bcf     STATUS , RP0    ;ram page 0
    return
;-------------------------
Delay       
    movlw   0x20    ;delay = 5N + 5 = 105us
    movwf   DelCount
DelLoop
    nop
    nop
    decfsz  DelCount
    goto    DelLoop
    return
;-------------------------
WriteTim                    ;Update 9852 on rising edge of ExtTim
    btfsc   ExtTim          ;Want a 0-1 transition, so need a low first
    goto    WriteTim
WaitTim
    btfss   ExtTim          ;Now wait for a high on external input
    goto    WaitTim         ;before updating DDS chip
    
Write9852                   ;Jump straight in here for immediate update
    bsf     IOUD
    nop
    nop
    nop
    bcf     IOUD
    movlw   "Z"
    call    SendAck
    return
;-------------------------
;------------------------
PushStack               ;Free up Stack1,  get rid of oldest
    movf    StackE, W
    movwf   StackF
    movf    StackD, W
    movwf   StackE
    movf    StackC, W
    movwf   StackD
    movf    StackB, W
    movwf   StackC
    movf    StackA, W
    movwf   StackB
    movf    Stack9, W
    movwf   StackA
    movf    Stack8, W
    movwf   Stack9
    movf    Stack7, W
    movwf   Stack8
    movf    Stack6, W
    movwf   Stack7
    movf    Stack5, W
    movwf   Stack6
    movf    Stack4, W
    movwf   Stack5
    movf    Stack3, W
    movwf   Stack4   
    movf    Stack2, W
    movwf   Stack3
    movf    Stack1, W
    movwf   Stack2
    return
;-----------------------
ClrStack
    clrf    StackF
    clrf    StackE
    clrf    StackD
    clrf    StackC
    clrf    StackB
    clrf    StackA
    clrf    Stack9
    clrf    Stack8
    clrf    Stack7
    clrf    Stack6
    clrf    Stack5
    clrf    Stack4
    clrf    Stack3
    clrf    Stack2
    clrf    Stack1
    return
;-----------------------    
CrLf
    movlw   0x0D
    call    Send232
    movlw   0x0A
    call    Send232
    return
;-----------------------
SetUpDDS
    movlw   7               ;Control register
    movwf   DDSAddr
    movlw   0x10
    movwf   D3
    movlw   0x64
    movwf   D2
    movlw   0x00            ;LSB of this is external update
    movwf   D1              ;Otherwise keep default values
    movlw   0x50
    movwf   D0

    call    DataTo9852
    call    Delay
    bsf     IOUD
    nop
    nop
    nop
    bcf     IOUD

    movlw   2
    movwf   DDSAddr
    movlw   0x40            ;Force Fout = 0.25.Fc
    movwf   D5
    clrf    D4
    clrf    D3
    clrf    D2
    clrf    D1
    clrf    D0

    call    DataTo9852
    call    Delay
    bsf     IOUD
    nop
    nop
    nop
    bcf     IOUD

    return
;--------------------------
Send9852Byte          
    movwf   Temp
    movlw   8
    movwf   Counter
DDSSendLoop
    rlf     Temp                  
    btfsc   STATUS, C
    bsf     SDIO
    btfss   STATUS, C
    bcf     SDIO
    nop
    bsf     SCLK
    nop
    bcf     SCLK

    decfsz  Counter
    goto    DDSSendLoop

    return
;-------------------------
SendAck
    call    Send232
    call    CrLf
    return
;==============================================
    org 0x2FA       ;Puts  all routines with tables and calculated jumps
                    ;   in page 3
;-----------------  
DataTo9852              ;Send contents of Dn...D0 to 9852
    movf    DDSAddr, W
    call    Send9852Byte        ;Send the address data

    call    GetRegLen           ;Returns with number of bytes to be sent
    movwf   Temp
    bcf     STATUS, C
    rlf     Temp, W
    sublw   d'12'               ;W = 12 - 2 * (No of bytes to be sent)
    addwf   PCL                 ;Jump into byte sending at the right place

    movf    D5, W
    call    Send9852Byte
    movf    D4, W
    call    Send9852Byte
    movf    D3, W
    call    Send9852Byte
    movf    D2, W
    call    Send9852Byte
    movf    D1, W
    call    Send9852Byte
    movf    D0, W
    call    Send9852Byte

    bcf     SDIO            ;Rest in low state for min. current consumption
    return
;-------------------------
DataToEE
    call    GetEEAddr           ;Returns with start address of this data item
    movwf   AutoEEADR
    call    GetRegLen
    movwf   Temp

    bcf     STATUS, C
    rlf     Temp, W             ;Multiply by 3
    addwf   Temp, W
    sublw   d'18'               ;W = 18 - 3 * (No of bytes to be sent)
    addwf   PCL                 ;Jump into byte storage at the right place

    movf    D5, W
    call    StoreEEW
    incf    AutoEEADR

    movf    D4, W
    call    StoreEEW
    incf    AutoEEADR

    movf    D3, W
    call    StoreEEW
    incf    AutoEEADR

    movf    D2, W
    call    StoreEEW
    incf    AutoEEADR

    movf    D1, W
    call    StoreEEW
    incf    AutoEEADR

    movf    D0, W
    call    StoreEEW
    incf    AutoEEADR

    return
;-------------------------
DispDData            ;Displays contents of D0-D5
    call    GetRegLen           ;Returns with number of bytes to be sent
    movwf   Temp
    bcf     STATUS, C
    rlf     Temp, W
    sublw   d'12'               ;W = 12 - 2 * (No of bytes to be sent)
    addwf   PCL                 ;Jump into byte sending at the right place

    movf    D5, W
    call    Hex232
    movf    D4, W
    call    Hex232
    movf    D3, W
    call    Hex232
    movf    D2, W
    call    Hex232
    movf    D1, W
    call    Hex232
    movf    D0, W
    call    Hex232

    return
;-------------------------
GetRegLen           ;gets number of bytes needed for value in DDSAddr
    movlw   HIGH($)            
    movwf   PCLATH             
    movf    DDSAddr, W
    addwf   PCL

    dt      2,2,6,6,6,4,3,4,2,2,1,2       ;Number of bytes in each register
;-------------------------
GetEEAddr           ;Get address of EE Start for each register's data
    movlw   HIGH($) ;  based on value in DDSAddr           
    movwf   PCLATH             
    movf    DDSAddr, W
    addwf   PCL
    
    dt      0,2,4,0x0A,0x10,0x16,0x1A,0x1D,0x21,0x23,0x25,0x26
;-------------------------
;Generating a message from data stored in programme memory
DispMsg   ;Enter with address of Text in W  eg.  'MOVLW LOW(TextLabel)'
    movwf   AddrHold    ;Store offset of text string

    movlw   HIGH(MsgTable)
    movwf   PCLATH

    movlw   low(MsgTable)
    addlw   1
    subwf   AddrHold , W    ;W = AddrTextString - TableStart - 1
                            ;  = Offset of string start in table
MsgLoop
    movwf   MsgCount
    call    MsgTable
    andlw   0xFF            ;end of message
    btfsc   STATUS , Z
    goto    MsgDone

    call    Send232
    movf    MsgCount, W
    addlw   1
    goto    MsgLoop

MsgDone
    call    CrLf
    return
;.........
MsgTable
    addwf   PCL
IntroMsg  dt "9852 DDS Controller G4JNT " , 0
RegPr1    dt "Reg No. " , 0
KPrompt   dt "Enter < 15 chars. of user data" , 0
EEFull    dt "OvFlo" , 0
BytesMsg  dt " Bytes of data",0
CmdLst    dt "Qxxxxxxxxxxxx[cr] Pxxxx[cr] Mxxxx[cr] U W V G K R",0
MsgTableEnd

    if ((DispMsg & 0xFF) >= (MsgTableEnd & 0xFF))
        ERROR    "Table code crosses page boundary"
        ERROR    "ADJUST START OF TABLE"
    endif

;-------------------------------    
    end
